#include "ofCvFloatImage.h"

#include "ofCvColorImage.h"
#include "ofCvGrayscaleImage.h"

//------------------------------------------------------------------------------------
ofCvFloatImage::ofCvFloatImage()
{
	bAllocated = false;
	width = height = 0;
}

//------------------------------------------------------------------------------------
ofCvFloatImage::ofCvFloatImage(int _width, int _height)
{
	bAllocated = false;
	allocate(_width, _height);
}

//------------------------------------------------------------------------------------
ofCvFloatImage::~ofCvFloatImage()
{
	clear();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::allocate(int _w, int _h){
	if (!bAllocated)
	{
		cvImage 			= cvCreateImage(cvSize(_w,_h), IPL_DEPTH_32F,1);
		cvImageTemp			= cvCreateImage(cvSize(_w,_h), IPL_DEPTH_32F,1);
		cvImageTemp2		= cvCreateImage(cvSize(_w,_h), IPL_DEPTH_32F,1);
		cvImage8U			= cvCreateImage(cvSize(_w,_h), IPL_DEPTH_8U,1);
		pixels 				= new unsigned char[_w*_h];
		width 				= _w;
		height 				= _h;
		tex.allocate(width, height, GL_LUMINANCE);
		bAllocated			= true;
	}
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::clear(){
	if (bAllocated)
	{
		cvReleaseImage(&cvImage);
		cvReleaseImage(&cvImageTemp);
		cvReleaseImage(&cvImageTemp2);
		cvReleaseImage(&cvImage8U);
		delete [] pixels;
		tex.clear();
		width = height = 0;
		bAllocated = false;
	}
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::set(int value){
	cvSet(cvImage, cvScalar(value));
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::setFromColorImage(ofCvColorImage& mom){
	cvCvtColor( mom.getCvImage() ,cvImage8U, CV_RGB2GRAY);
	cvConvertScale(cvImage8U, cvImage, 1, 0);
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::setFromGrayscaleImage(ofCvGrayscaleImage& mom){	
	cvConvertScale( mom.getCvImage(), cvImage, 1, 0);
}


//-------------------------------------------------------------------------------------
void ofCvFloatImage::setFromPixels(unsigned char * _pixels, int w, int h){
	for (int i = 0; i < h; i++){
		memcpy(cvImage->imageData + (i * cvImage->widthStep), _pixels + (i * w), w);
	}
	for (int i = 0; i < h; i++){
	  memcpy(pixels + (i * w), cvImage->imageData + (i * cvImage->widthStep), w);
	}
}

//-------------------------------------------------------------------------------------
unsigned char * ofCvFloatImage::getPixels(){
		// copy each line of pixels: 
    cvConvertScale( cvImage, cvImage8U, 1, 0);
    for (int i = 0; i < height; i++){
		memcpy(pixels + (i * width), cvImage8U->imageData + (i * cvImage8U->widthStep), width);
	}	
	return pixels;
}


void ofCvFloatImage::draw(float x, float y){

	// note, this is a bit ineficient, as we have to
	// copy the data out of the cvImage into the pixel array
	// and then upload to texture.  We should add
	// to the texture class an override for pixelstorei
	// that allows stepped-width image upload:

	tex.loadData(getPixels(), width, height, GL_LUMINANCE);
	tex.draw(x,y,width, height);
}

void ofCvFloatImage::draw(float x, float y, float w, float h){

	// note, this is a bit ineficient, as we have to
	// copy the data out of the cvImage into the pixel array
	// and then upload to texture.  We should add
	// to the texture class an override for pixelstorei
	// that allows stepped-width image upload:

	tex.loadData(getPixels(), width, height, GL_LUMINANCE);
	tex.draw(x,y,w,h);
}


//-------------------------------------------------------------------------------------
void ofCvFloatImage::swapTemp(){
	IplImage * temp;
	temp 			= cvImage;
	cvImage 		= cvImageTemp;
	cvImageTemp 	= temp;
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::contrastStretch(){
	double minVal, maxVal;
	cvMinMaxLoc(cvImage, &minVal, &maxVal, NULL, NULL, 0);
	double scale=1.0f;
	double shift=0;
	if ((maxVal-minVal) != 0){
		scale=255.0/(maxVal-minVal);
    	shift=-minVal*scale;
	}
	cvConvertScale(cvImage, cvImageTemp, scale, shift);
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::threshold(int value){
	//http://lush.sourceforge.net/lush-manual/01a8321b.html
	cvThreshold(cvImage,cvImageTemp,value,255,CV_THRESH_BINARY);
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::dilate_3x3(){
	cvDilate( cvImage, cvImageTemp, 0, 1 );
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::erode_3x3(){
	cvErode( cvImage, cvImageTemp, 0, 1 );
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::blur(){
	cvSmooth(cvImage, cvImageTemp, CV_GAUSSIAN , 3);
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::blurHeavily(){
	cvSmooth(cvImage, cvImageTemp, CV_GAUSSIAN , 33);
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::absDiff(ofCvFloatImage& mom, ofCvFloatImage& dad){
	cvAbsDiff(mom.getCvImage(), dad.getCvImage(),cvImage);
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::mirror(bool bFlipVertically, bool bFlipHorizontally){
	int flipMode = 0;
	//---------------------------------
	if (bFlipVertically && !bFlipHorizontally) flipMode = 0;
	else if (!bFlipVertically && bFlipHorizontally) flipMode = 1;
	else if (bFlipVertically && bFlipHorizontally) flipMode = -1;
	else return;
	//---------------------------------
	cvFlip(cvImage, cvImageTemp, flipMode);
	swapTemp();
}

// custom filter by cj
void
ofCvFloatImage::filter(CvMat* pFilter)
{
	cvFilter2D(cvImage, cvImageTemp, pFilter);

    double scale, shift;
    double minVal, maxVal;

    cvMinMaxLoc(cvImageTemp,  &minVal, &maxVal);
    scale = 255/(maxVal - minVal);
    shift = -minVal*scale;

	cvConvertScale(cvImageTemp, cvImage, scale, shift);
}

/*
void
ofCvFloatImage::sobel(int iXOrder, int iYOrder, int iAperatureSize)
{
	cvSobel(cvImage, cvImageTempFilter, iXOrder, iYOrder, iAperatureSize);
	
    double scale, shift;
    double minVal, maxVal;

    cvMinMaxLoc(cvImageTempFilter,  &minVal, &maxVal);
    scale = 255/(maxVal - minVal);
    shift = -minVal*scale;

	cvConvertScale(cvImageTempFilter, cvImage, scale, shift);
}
*/


//-------------------------------------------------------------------------------------
void ofCvFloatImage::warpIntoMe(ofCvFloatImage& mom, ofCvPoint2f src[4], ofCvPoint2f dst[4]){
	// compute matrix for perspectival warping (homography)
	CvPoint2D32f cvsrc[4];
	CvPoint2D32f cvdst[4];
	CvMat *translate = cvCreateMat(3,3,CV_32FC1);
	cvSetZero(translate);
	for (int i = 0; i < 4; i++){
		cvsrc[i].x = src[i].x;
		cvsrc[i].y = src[i].y;
		cvdst[i].x = dst[i].x;
		cvdst[i].y = dst[i].y;
	}
	cvWarpPerspectiveQMatrix(cvsrc, cvdst, translate);  // calculate homography
	cvWarpPerspective( mom.getCvImage(), cvImage, translate);
	cvReleaseMat(&translate);
}

//-------------------------------------------------------------------------------------
int ofCvFloatImage::countNonZeroInRegion(int x, int y, int w, int h){

	int count = 0;
	cvSetImageROI(cvImage, cvRect(x,y,w,h));
	count = cvCountNonZero(cvImage);
	cvResetImageROI( cvImage );
	return count;
}


//-------------------------------------------------------------------------------------
void ofCvFloatImage::operator =	( ofCvFloatImage & mom ){
	if (mom.width == width && mom.height == height){
		cvCopy(mom.getCvImage(), cvImage, 0);
	} else {
		// what do we do?
		// cvResize?
		// cvClone?
		printf("error in =, images are different sizes\n");
	}
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::operator -=	(  ofCvFloatImage & mom ){
	if (mom.width == width && mom.height == height){
		cvSub(cvImage, mom.getCvImage(), cvImageTemp);
		swapTemp();
	} else {
		printf("error in -=, images are different sizes\n");
	}
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::operator +=	(  ofCvFloatImage & mom ){
	if (mom.width == width && mom.height == height){
		cvAdd(cvImage, mom.getCvImage(), cvImageTemp);
		swapTemp();
	} else {
		// what do we do?
		// cvResize?
		// cvClone?
		printf("error in =, images are different sizes\n");
	}
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::operator *=	(  ofCvFloatImage & mom ){
	if (mom.width == width && mom.height == height){
		float scalef = 1.0f / 255.0f;
		cvMul(cvImage, mom.getCvImage(), cvImageTemp, scalef);
		swapTemp();
	} else {
		// what do we do?
		// cvResize?
		// cvClone?
		printf("error in =, images are different sizes\n");
	}
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::operator -=	( float scalar ){
	cvSubS(cvImage, cvScalar(scalar), cvImageTemp);
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::operator +=	( float scalar ){
	cvAddS(cvImage, cvScalar(scalar), cvImageTemp);
	swapTemp();
}

//-------------------------------------------------------------------------------------
void ofCvFloatImage::addWeighted( ofCvGrayscaleImage& mom, float f)
{
     cvConvertScale( mom.getCvImage(), cvImageTemp2, 1, 0);
     
     cvAddWeighted( cvImageTemp2, f, cvImage, 1.0f-f, 0, cvImageTemp);
     swapTemp();
}